@@ -1,4 +1,5 @@
-from dataclasses import Field, dataclass
+from collections import defaultdict
+from dataclasses import Field
 from functools import wraps
 from inspect import Parameter, isgeneratorfunction, signature
 from types import MethodType
@@ -7,8 +8,10 @@ from typing import (
     Any,
     Callable,
     Collection,
+    Dict,
     Iterable,
     Iterator,
+    List,
     Optional,
     Sequence,
     Type,
@@ -17,8 +20,10 @@ from typing import (
     overload,
 )
 
-from apischema.types import MetadataMixin
-from apischema.utils import PREFIX
+from apischema.conversions.dataclass_models import get_model_origin, has_model_origin
+from apischema.types import AnyType
+from apischema.typing import get_type_hints
+from apischema.utils import get_origin_or_type, is_method, method_class
 from apischema.validation.dependencies import find_all_dependencies
 from apischema.validation.errors import (
     Error,
@@ -29,8 +34,19 @@ from apischema.validation.errors import (
 )
 from apischema.validation.mock import NonTrivialDependency
 
-# use attribute instead of global dict in order to be inherited
-VALIDATORS_ATTR = f"{PREFIX}validators"
+_validators: Dict[Type, List["Validator"]] = defaultdict(list)
+
+
+def get_validators(tp: AnyType) -> Sequence["Validator"]:
+    validators = []
+    if hasattr(tp, "__mro__"):
+        for sub_cls in tp.__mro__:
+            validators.extend(_validators[sub_cls])
+    else:
+        validators.extend(_validators[tp])
+    if has_model_origin(tp):
+        validators.extend(get_validators(get_model_origin(tp)))
+    return validators
 
 
 class Discard(Exception):
@@ -91,6 +107,7 @@ class Validator:
                     raise Discard(self.discard, err)
 
         self.validate: Callable[..., Iterator[Error]] = validate
+        self._registered = False
 
     def __get__(self, instance, owner):
         return self if instance is None else MethodType(self.func, instance)
@@ -98,46 +115,15 @@ class Validator:
     def __call__(self, *args, **kwargs):
         return self.func(*args, **kwargs)
 
-    def __set_name__(self, owner, name):
+    def _register(self, owner: Type):
+        if self._registered:
+            raise RuntimeError("Validator already registered")
         self.dependencies = find_all_dependencies(owner, self.func) | self.params
-        setattr(owner, VALIDATORS_ATTR, (*get_validators(owner), self))
+        _validators[owner].append(self)
+        self._registered = True
 
-
-Func = TypeVar("Func", bound=Callable)
-
-
-def get_validators(obj) -> Sequence[Validator]:
-    return getattr(obj, VALIDATORS_ATTR, ())
-
-
-@overload
-def add_validator(
-    cls: Type, *, field: Field = None, discard: Collection[Field] = None
-) -> Callable[[Func], Func]:
-    ...
-
-
-@overload
-def add_validator(
-    cls: Type, *funcs: Callable, field: Field = None, discard: Collection[Field] = None
-):
-    ...
-
-
-def add_validator(
-    cls: Type, *funcs: Callable, field: Field = None, discard: Collection[Field] = None
-):
-    if not funcs:
-
-        def wrapper(func: Func) -> Func:
-            add_validator(cls, func, field=field, discard=discard)
-            return func
-
-        return wrapper
-
-    for func in funcs:
-        validator = Validator(func, field, discard)
-        validator.__set_name__(cls, func.__name__)
+    def __set_name__(self, owner, name):
+        self._register(owner)
 
 
 T = TypeVar("T")
@@ -145,9 +131,7 @@ T = TypeVar("T")
 
 def validate(__obj: T, __validators: Iterable[Validator] = None, **kwargs) -> T:
     if __validators is None:
-        __validators = get_validators(__obj)
-    if not __validators:
-        return __obj
+        __validators = get_validators(type(__obj))
     error: Optional[ValidationError] = None
     __validators = iter(__validators)
     while True:
@@ -192,15 +176,30 @@ def validator(func: V) -> V:
 
 
 @overload
-def validator(field: Any = None, *, discard: Any = None) -> Callable[[V], V]:
+def validator(
+    field: Any = None, *, discard: Any = None, owner: Type = None
+) -> Callable[[V], V]:
     ...
 
 
-def validator(arg=None, *, discard=None):
-    if callable(arg):
-        return Validator(arg, None, discard)
+def validator(arg=None, *, discard=None, owner=None):
     if arg is None:
-        return lambda func: validator(func, discard=discard)
+        return lambda func: Validator(func, None, discard)
+    if callable(arg):
+        validator_ = Validator(arg, None, None)
+        if is_method(arg) and method_class(arg) is None:
+            return validator_
+        if is_method(arg):
+            if owner is None:
+                owner = method_class(arg)
+        if owner is None:
+            try:
+                first_param = next(iter(signature(arg).parameters))
+                owner = get_origin_or_type(get_type_hints(arg)[first_param])
+            except Exception:
+                raise ValueError("Validator first parameter must be typed")
+        validator_._register(owner)
+        return validator_
     field = arg
     if not isinstance(field, Field):
         raise TypeError(
@@ -213,17 +212,3 @@ def validator(arg=None, *, discard=None):
         ):
             raise TypeError("discard must be a field or a collection of fields")
     return lambda func: Validator(func, field, discard)
-
-
-@dataclass(frozen=True)
-class ValidatorsMetadata(MetadataMixin):
-    validators: Sequence[Validator]
-
-    def __post_init__(self):
-        from apischema.metadata.keys import VALIDATORS_METADATA
-
-        MetadataMixin.__init__(self, VALIDATORS_METADATA)
-
-
-def validators(*validator: Callable) -> ValidatorsMetadata:
-    return ValidatorsMetadata(list(map(Validator, validator)))
